
import enum

from AgentService.utils import now
from AgentService.dtypes.db import DatabaseItem


class ChatStatus(enum.Enum):
    created = "created"
    error = "error"
    idle = "idle"
    tools = "tools"
    generating = "generating"
    finished = "finished"


class Chat(DatabaseItem):
    def __init__(
        self,
        id: str,
        chat_id: str,
        status: ChatStatus | str,
        created: int = None,
        data: dict = None
    ):

        self.id = id
        self.chat_id = chat_id
        self.status = status if isinstance(status, ChatStatus) else ChatStatus(status)
        self.data = data
        self.created = created if created else now()

        self.fields = ["id", "chat_id", "status", "data", "created"]




from pydantic import BaseModel


class AgentTool:
    name: str = "tool"
    description: str = "this is tool"
    parameters: BaseModel = None

    @property
    def to_schema(self) -> dict:
        return {
            "name": self.name,
            "description": self.description,
            "parameters": self.parameters.model_json_schema() if self.parameters else {},
        }




import typing

import motor.motor_asyncio as motor
from motor.motor_asyncio import AsyncIOMotorDatabase, AsyncIOMotorCollection

from loguru import logger as log

from AgentService.utils.singleton import SingletonMeta
from AgentService.dtypes.db import DatabaseItem
from AgentService.dtypes.db import method as mth
from AgentService.dtypes.response import BaseResponse, ErrResponse, OkResponse


class Db(metaclass=SingletonMeta):
    def __init__(self):
        self.classes = {}

        self.log = log.bind(classname=self.__class__.__name__)
        self.client: motor.AsyncIOMotorClient = None
        self.db: AsyncIOMotorDatabase = None

    def items(self) -> list[DatabaseItem]:
        res = []

        for obj in self.classes:
            if not isinstance(obj, str):
                res.append(obj)

        return res

    async def ex(
            self,
            method: typing.Type[mth.BaseDatabaseMethod]
    ) -> typing.Type[BaseResponse]:
        response = await self._ex(method)

        if response.status != "ok":
            self.log.warning(response)
            return None

        return response.data

    async def _ex(
            self,
            method: typing.Type[mth.BaseDatabaseMethod]
    ) -> typing.Type[BaseResponse]:

        if self.db is None:
            raise RuntimeError("Can't execute database method without connection")

        class_methods = self.classes.get(method.target)

        if not class_methods:
            raise NotImplementedError(f"{method.target.__name__} hasn't been added to db")

        if method.__class__ not in class_methods["methods"]:
            raise NotImplementedError(f"There is no {method.__class__.__name__} method for {method.target.__name__}")

        item_class = class_methods["class"]
        collection_name = item_class.__name__

        try:
            match method.name:
                case "add_one":
                    return await self.__add_one(collection_name, method.data)

                case "add_many":
                    return await self.__add_many(collection_name, method.data)

                case "get_one":
                    return await self.__get_one(collection_name, method.data, item_class)

                case "get_many":
                    return await self.__get_many(collection_name, method.data, item_class)

                case "update_one":
                    return await self.__update_one(collection_name, method.data, method.to_update)

                case "update_many":
                    return await self.__update_many(collection_name, method.data)

                case "remove_one":
                    return await self.__remove_one(collection_name, method.data)

                case "remove_many":
                    return await self.__remove_many(collection_name, method.data)

                case unsupported_method:
                    raise NotImplementedError(f"There is not such method {unsupported_method}")

        except Exception as err:
            self.log.exception(err)
            self.log.error(f"Cant execute {method}")

    def connect(
        self,
        name: str,
        uri: str
    ):
        if self.client:
            self.log.warning(f"Already connected to db")
            return

        for item in DatabaseItem.__subclasses__():
            self.__add_methods(DatabaseItem.get_methods(item))

        try:
            self.client = motor.AsyncIOMotorClient(uri)
            self.db = self.client[name]
            self.log.info("Connected to db")

        except Exception as err:
            self.log.exception(err)
            self.log.critical(f"Cant connect to db")
            exit()

    def __add_methods(
            self,
            class_method
    ):

        self.classes.update(class_method)
        self.log.debug(f"Added new methods {class_method}")

    async def __is_item(
            self,
            collection_name: str,
            item_id: str
    ) -> bool:

        collection: AsyncIOMotorCollection = self.db[collection_name]
        item = await collection.find_one({"id": item_id}, {"_id": 0})
        return not item is None

    async def __is_items(
            self,
            collection_name: str,
            item_ids: list[str]
    ) -> dict[str, bool]:
        resp = {
            item_id: False
            for item_id in item_ids
        }

        collection: AsyncIOMotorCollection = self.db[collection_name]
        items = collection.find({"id": {"$in": item_ids}}, {"_id": 0})

        async for i in items:
            resp.update({
                i['id']: True
            })

        return resp

    async def __add_one(
            self,
            collection_name: str,
            item: typing.Type[DatabaseItem]
    ) -> typing.Type[BaseResponse]:

        if not isinstance(item, DatabaseItem):
            return ErrResponse(f"item have to be type of {DatabaseItem}", data=item)

        if await self.__is_item(collection_name, item.id):
            return ErrResponse(f"Duplicate item: {item}", data=item)

        collection: AsyncIOMotorCollection = self.db[collection_name]
        response = await collection.insert_one(item.to_dict())

        return OkResponse(data={"db_answer": str(response)})

    async def __add_many(
            self,
            collection_name: str,
            items: typing.Type[DatabaseItem] | list[typing.Type[DatabaseItem]]
    ) -> typing.Type[BaseResponse]:

        if type(items) not in [list, tuple]:
            items = [items]

        for item in items:
            if not isinstance(item, DatabaseItem):
                return ErrResponse(f"Every item have to be type of {DatabaseItem}", data=items)

        is_items = await self.__is_items(collection_name, list(map(lambda x: x.id, items)))

        to_add = []
        for item in items:
            if is_items[item.id]:
                continue

            to_add.append(item.to_dict())

        if not len(to_add):
            return OkResponse(
                description=f"Added 0 items, duplicate: {len(items)} items"
            )

        collection: AsyncIOMotorCollection = self.db[collection_name]
        resps = []

        for row in to_add:
            try:
                response = await collection.insert_one(row)
                resps.append(response)

            except Exception as err:
                self.log.exception(err)
                resps.append(str(err))

        return OkResponse(
            description=f"Added {len(to_add)} items, duplicate: {len(items)-len(to_add)} items",
            data={
                "db_answers": resps
            }
        )

    async def __get_one(
            self,
            collection_name: str,
            query: typing.Type[DatabaseItem] | dict, item_class: type
    ) -> typing.Type[BaseResponse]:
        if isinstance(query, DatabaseItem):
            query = query.to_dict()

        collection: AsyncIOMotorCollection = self.db[collection_name]

        item = await collection.find_one(query, {"_id": 0})

        if not item:
            return OkResponse(description=f"No item found with {query = }")

        return OkResponse(data=item_class(**item))

    async def __get_many(
            self,
            collection_name: str,
            query: typing.Type[DatabaseItem] | dict,
            item_class: type
    ) -> typing.Type[BaseResponse]:
        if isinstance(query, DatabaseItem):
            query = query.to_dict()

        collection: AsyncIOMotorCollection = self.db[collection_name]
        items = collection.find(query, {"_id": 0})

        data = []
        async for i in items:
            data.append(item_class(**i))

        return OkResponse(data=data)

    async def __update_one(
            self,
            collection_name: str,
            item: typing.Type[DatabaseItem],
            to_update: list[str] | None
    ) -> typing.Type[BaseResponse]:

        if not isinstance(item, DatabaseItem):
            return ErrResponse(f"item have to be type of {DatabaseItem}", data=item)

        if not await self.__is_item(collection_name, item.id):
            return ErrResponse(f"There is not such item to update: {item}", data=item)

        collection: AsyncIOMotorCollection = self.db[collection_name]

        data = item.to_dict()
        if to_update:
            for field in list(data):
                if field not in to_update:
                    del data[field]

        response = await collection.update_one({"id": item.id}, {"$set": data})

        return OkResponse(data={"db_answer": str(response)})

    async def __update_many(
            self,
            collection_name: str,
            item: typing.Type[DatabaseItem]
    ) -> typing.Type[BaseResponse]:

        raise NotImplementedError()

    async def __remove_one(
            self,
            collection_name: str,
            item: typing.Type[DatabaseItem]
    ) -> typing.Type[BaseResponse]:

        if not isinstance(item, DatabaseItem):
            return ErrResponse(f"item have to be type of {DatabaseItem}", data=item)

        if not await self.__is_item(collection_name, item.id):
            return ErrResponse(f"There is not such item to remove: {item}", data=item)

        collection: AsyncIOMotorCollection = self.db[collection_name]
        response = await collection.delete_one({"id": item.id})

        return OkResponse(data={"db_answer": str(response)})

    async def __remove_many(
            self,
            collection_name: str,
            item: typing.Type[DatabaseItem]
    ) -> typing.Type[BaseResponse]:

        raise NotImplementedError()




import functools
import asyncio
import json
import typing
import aiohttp

from loguru import logger

from .event import ToolEvent

from AgentService.agent.response import AgentResponse, AgentResponseType
from AgentService.dtypes.message import SendMessageRequest, Tool, ToolAnswer


class AgentConnector:
    def __init__(
        self,
        endpoint: str,
        key: str = None
    ):

        self.log = logger.bind(classname=self.__class__.__name__)

        self.endpoint = endpoint[:-1] if endpoint.endswith("/") else endpoint
        self.key = key

        self.callbacks = {}

    def bind_tool_output(self, tool_name: str, function: typing.Callable):
        self.callbacks = {
            tool_name: functools.partial(self.answer_tool, function=function)
        }

    async def answer_tool(self, event: ToolEvent, function: typing.Callable) -> ToolAnswer:
        try:
            response = await function(data=event.tool.arguments, chat_id=event.chat_id)

            if response in [None, ""]:
                response = "function returned nothing"

            elif not isinstance(response, str):
                response = str(response)

            return ToolAnswer(
                id=event.tool.id,
                name=event.tool.name,
                text=response
            )

        except Exception as err:
            self.log.exception(err)

            return ToolAnswer(
                id=event.tool.id,
                name=event.tool.name,
                text="function returned nothing"
            )

    async def dispatch_event(self, event: ToolEvent):
        self.log.debug(f"Dispatching event -> {event}")

        callback = self.callbacks.get(event.tool.name)

        self.log.debug(f"Got callback -> {callback}")

        if not callback:
            return ToolAnswer(
                id=event.tool.id,
                name=event.tool.name,
                text="function not binded"
            )

        return await callback(event=event)

    async def handle_tools(self, tools: list[Tool], chat_id) -> list[ToolAnswer]:
        tool_answers = [
            await self.dispatch_event(ToolEvent(tool_name=tool.name, tool=tool, chat_id=chat_id))
            for tool in tools
        ]

        return tool_answers

    async def request(self, request: SendMessageRequest) -> AgentResponse:
        self.log.debug(f"Sending request -> {request.to_dict()}")

        async with aiohttp.ClientSession() as session:
            async with session.post(
                url=f"{self.endpoint}/chat",
                json=request.to_dict(),
                raise_for_status=False
            ) as resp:

                self.log.debug(f"Got response -> {await resp.text()}")

                response = await resp.json()
                agent_response = AgentResponse(**response["data"])

                return agent_response

    async def send(self, chat_id: str, text: str = None, context: dict = {}, tool_answers: list[ToolAnswer] = []) -> AgentResponse:
        agent_response = await self.request(SendMessageRequest(chat_id=chat_id, text=text, context=context, tool_answers=tool_answers))

        if agent_response.type == AgentResponseType.answer:
            return agent_response

        tool_answers = await self.handle_tools(agent_response.content, chat_id)

        return await self.send(chat_id, tool_answers=tool_answers)




import enum

from AgentService.utils import now
from AgentService.dtypes.db import DatabaseItem

from AgentService.utils.jsonify import Jsonified


class MessageType(enum.Enum):
    system = "system"
    user = "user"
    assistant = "assistant"
    assistant_skip = "assistant_skip"
    tools = "tools"
    tool_answer = "tool_answer"


class Tool(Jsonified):
    def __init__(
        self,
        id: str,
        name: str,
        arguments: str
    ):

        self.id = id
        self.name = name
        self.arguments = arguments

        self.fields = ["id", "name", "arguments"]


class ToolAnswer(Jsonified):
    def __init__(
        self,
        id: str,
        name: str,
        text: str
    ):

        self.id = id
        self.name = name
        self.text = text

        self.fields = ["id", "name", "text"]


class SendMessageRequest(Jsonified):
    def __init__(
        self,
        chat_id: str,
        text: str = None,
        context: dict = {},
        tool_answers: list[ToolAnswer] = []
    ):

        self.chat_id = chat_id
        self.text = text
        self.context = context
        self.tool_answers = tool_answers

        self.fields = ["chat_id", "text", "context", "tool_answers"]


class Message(DatabaseItem):
    def __init__(
        self,
        id: str,
        chat_id: str,
        type: MessageType | str,
        text: str = None,
        tools: list[Tool] = None,
        tool_answer: ToolAnswer = None,
        created: int = None,
    ):

        if isinstance(tool_answer, dict):
            tool_answer = ToolAnswer(**tool_answer)

        self.id = id
        self.chat_id = chat_id
        self.text = text
        self.type = type if isinstance(type, MessageType) else MessageType(type)
        self.tools = list(map(lambda x: x if isinstance(x, Tool) else Tool(**x), tools)) if tools else []
        self.tool_answer = tool_answer
        self.created = created if created else now()

        self.fields = ["id", "chat_id", "text", "type", "tools", "tool_answer", "created"]

    @property
    def gpt_dump(self) -> dict:
        if self.type == MessageType.tools:
            return {
                "role": "assistant",
                "tool_calls": [
                    {
                        "id": tool.id,
                        "type": "function",
                        "function": {
                            "name": tool.name,
                            "arguments": tool.arguments
                        }
                    }
                    for tool in self.tools
                ]
            }

        elif self.type == MessageType.tool_answer:
            return {
                "role": "tool",
                "content": self.tool_answer.text,
                "tool_call_id": self.tool_answer.id,
                "name": self.tool_answer.name,
            }

        elif self.type == MessageType.assistant_skip:
            return {
                "role": "assistant",
                "content": self.text
            }

        else:
            return {
                "role": self.type.value,
                "content": self.text
            }




import configparser
import os
import importlib

from AgentService.utils.singleton import SingletonMeta
from AgentService.agent import Agent


class Config(metaclass=SingletonMeta):
    project_name: str

    log_level: str
    log_path: str

    agent: Agent

    def __init__(self, project_path):
        config_path = os.path.join(project_path, "agent.cfg")
        config = configparser.ConfigParser()
        config.read(config_path)

        self.open_ai_key = os.getenv("openai_key")
        self.db_name = os.getenv("db_name", default="AgentService")
        self.db_uri = os.getenv("db_uri")

        self.project_name = config["project"]["name"]

        self.agent_path = config["project"]["agent_source"]
        importlib.import_module(self.agent_path)

        self.tools_path = config["project"]["tools_source"]
        importlib.import_module(self.tools_path)

        self.app_host = config["app"]["host"]
        self.app_port = int(config["app"]["port"])

        self.log_level = config["logging"]["level"]
        self.log_path = config["logging"]["path"]

        self.agent = None

    def set_agent(self, agent: Agent):
        self.agent = agent




from loguru import logger
from uuid import uuid4
from openai import AsyncOpenAI

from .agent_tool import AgentTool
from .response import AgentResponse, AgentResponseType

from AgentService.db import Db
from AgentService.dtypes.db import method as dmth
from AgentService.dtypes.chat import Chat, ChatStatus
from AgentService.dtypes.message import Message, MessageType, ToolAnswer, Tool


class Agent:
    model: str = "gpt-4.1-nano"
    temperature: float = 1.0
    max_tokens: int = 2048
    top_p: float = 1.0

    system_prompt: str = "You are a helpful assistant"
    prompt: str = "{text}"

    is_one_shot: bool = False

    def __init__(
        self,
        openai_key: str,
        tools: list[AgentTool] = None,
    ):

        self.log = logger.bind(classname=self.__class__.__name__)
        self.db = Db()
        self.client = AsyncOpenAI(api_key=openai_key)

        self.tools = tools if tools else []
        self.tools_schema = [{"type": "function", "function": tool.to_schema} for tool in self.tools]

        self.log.debug(f"Created gpt like tools -> {self.tools_schema}")

    async def __system_prompt(self, context: dict) -> str:
        return self.system_prompt.format(**context)

    async def __prompt(self, text: str, context: dict) -> str:
        return self.prompt.format(text=text, **context)

    async def __generate(self, chat: Chat) -> AgentResponse:
        chat.status = ChatStatus.generating
        await self.db.ex(dmth.UpdateOne(Chat, chat, to_update="status"))

        messages: list[Message] = await self.db.ex(dmth.GetMany(Message, chat_id=chat.id))
        gpt_messages = list(map(lambda x: x.gpt_dump, messages))

        if len(self.tools_schema):
            response = await self.client.chat.completions.create(
                model=self.model,
                messages=gpt_messages,
                tools=self.tools_schema,
                tool_choice="auto"
            )

        else:
            response = await self.client.chat.completions.create(
                model=self.model,
                messages=gpt_messages
            )

        message = response.choices[0].message
        if message.tool_calls:
            return AgentResponse(
                type=AgentResponseType.tools,
                content=[
                    Tool(
                        id=tool_call.id,
                        name=tool_call.function.name,
                        arguments=tool_call.function.arguments
                    )
                    for tool_call in message.tool_calls
                ]
            )

        return AgentResponse(
            type=AgentResponseType.answer,
            content=message.content
        )

    async def generate(self, chat: Chat) -> AgentResponse:
        try:
            return await self.__generate(chat)

        except Exception as err:
            self.log.exception(err)

            return AgentResponse(
                type=AgentResponseType.answer,
                content=str(err)
            )

    async def get_chat(self, chat_id: str, context: dict = {}) -> Chat:
        chat: Chat = await self.db.ex(dmth.GetOne(
            Chat,
            chat_id=chat_id,
            status={
                "$in": [ChatStatus.created.value, ChatStatus.idle.value, ChatStatus.tools.value]
            }
        ))
        if not chat:
            chat = Chat(
                id=uuid4().hex,
                chat_id=chat_id,
                status=ChatStatus.created,
                data=context
            )
            await self.db.ex(dmth.AddOne(Chat, chat))

        system_message: Message = await self.db.ex(dmth.GetOne(Message, chat_id=chat.id, type=MessageType.system.value))
        if not system_message:
            system_message = Message(
                id=uuid4().hex,
                chat_id=chat.id,
                text=await self.__system_prompt(context),
                type=MessageType.system
            )
            await self.db.ex(dmth.AddOne(Message, system_message))

        return chat

    async def proccess_answer(self, answer: AgentResponse, chat: Chat) -> AgentResponse:
        if answer.type == AgentResponseType.tools:
            bot_message = Message(
                id=uuid4().hex,
                chat_id=chat.id,
                type=MessageType.tools,
                tools=answer.content
            )
            chat.status = ChatStatus.tools

        elif answer.type == AgentResponseType.answer:
            bot_message = Message(
                id=uuid4().hex,
                chat_id=chat.id,
                type=MessageType.assistant,
                text=answer.content
            )
            chat.status = ChatStatus.finished if self.is_one_shot else ChatStatus.idle

        else:
            raise ValueError(f"wrong answer type got {answer.type}, expected {AgentResponseType} like")

        await self.db.ex(dmth.AddOne(Message, bot_message))
        await self.db.ex(dmth.UpdateOne(Chat, chat, to_update="status"))

        return answer

    async def skip_tools(self, chat) -> str:
        messages: list[Message] = await self.db.ex(dmth.GetMany(Message, chat_id=chat.id))
        last_message: Message = messages[-1]

        if not last_message.tools:
            chat.status = ChatStatus.idle
            await self.db.ex(dmth.UpdateOne(Chat, chat, to_update="status"))
            return chat

        new_messages = [
            Message(
                id=uuid4().hex,
                chat_id=chat.id,
                type=MessageType.tool_answer,
                tool_answer=ToolAnswer(
                    id=tool.id,
                    name=tool.name,
                    text="Tool call skipped."
                )
            )
            for tool in last_message.tools
        ] + [
            Message(
                id=uuid4().hex,
                chat_id=chat.id,
                type=MessageType.assistant,
                text="skip"
            )
        ]
        await self.db.ex(dmth.AddMany(Message, new_messages))

        return chat

    async def answer_text(self, chat_id: str, text: str, context: dict = {}) -> AgentResponse:
        chat = await self.get_chat(chat_id, context)
        if chat.status == ChatStatus.tools:
            chat = await self.skip_tools(chat)

        user_message = Message(
            id=uuid4().hex,
            chat_id=chat.id,
            text=await self.__prompt(text, context),
            type=MessageType.user
        )
        await self.db.ex(dmth.AddMany(Message, user_message))

        answer = await self.generate(chat=chat)
        await self.proccess_answer(answer, chat)

        return answer

    async def answer_tools(self, chat_id: str, tool_answers: list[ToolAnswer] = None) -> AgentResponse:
        chat = await self.get_chat(chat_id)

        if chat.status != ChatStatus.tools:
            return AgentResponse(type=AgentResponseType.answer, content="No tools to answer")

        new_messages = []
        for tool_answer in tool_answers:
            new_messages.append(Message(
                id=uuid4().hex,
                chat_id=chat.id,
                type=MessageType.tool_answer,
                tool_answer=tool_answer
            ))
        await self.db.ex(dmth.AddMany(Message, new_messages))

        answer = await self.generate(chat=chat)
        await self.proccess_answer(answer, chat)

        return answer

    async def answer(self, chat_id: str, text: str = None, context: dict = {}, tool_answers: list[ToolAnswer] = None) -> AgentResponse:
        self.log.debug(f"Answer: {chat_id = }, {text = }, {context = }, {tool_answers = }")

        if text:
            return await self.answer_text(chat_id, text, context)

        elif len(tool_answers):
            return await self.answer_tools(chat_id, tool_answers)

        else:
            raise ValueError("Need text or tool answers to answer")




import fastapi
import uvicorn

from AgentService.config import Config

from .chat import chat_router


app = fastapi.FastAPI()
app.include_router(chat_router)


def start_app():
    config = Config()
    uvicorn.run(app, host=config.app_host, port=config.app_port)




from . import method
from AgentService.utils.jsonify import Jsonified


class DatabaseItem(Jsonified):
    id: str
    methods: list[method.BaseDatabaseMethod] = [
        method.GetOne, method.GetMany,
        method.AddOne, method.AddMany,
        method.UpdateOne, method.UpdateMany,
        method.RemoveOne, method.RemoveMany
    ]

    @staticmethod
    def get_methods(item) -> dict:
        if not issubclass(item, DatabaseItem):
            raise TypeError(f"Can't get db methods from not DatabaseItem type, got {item}")

        return {
            item.__name__: {
                "methods": item.methods,
                "class": item
            },
            item: {
                "methods": item.methods,
                "class": item
            }
        }

    def __str__(self):
        return f"{self.__class__.__name__}(id='{self.id}')"




from AgentService.utils.jsonify import BaseJsonifed


class BaseDatabaseMethod:
    name = "base"
    target: str
    data: object | dict
    to_update: list[str] | None

    def __init__(self, target, data=None, to_update=None, **kwargs):
        self.target = target
        self.to_update = to_update
        self.data = data if data else {}

        if isinstance(self.data, dict):
            unpacked = {}

            for key in kwargs:
                item = kwargs[key]

                if issubclass(item.__class__, BaseJsonifed):
                    item = item.to_dict()

                unpacked.update({key: item})

            self.data.update(unpacked)


class AddOne(BaseDatabaseMethod):
    name = "add_one"


class AddMany(BaseDatabaseMethod):
    name = "add_many"


class GetOne(BaseDatabaseMethod):
    name = "get_one"


class GetMany(BaseDatabaseMethod):
    name = "get_many"


class UpdateOne(BaseDatabaseMethod):
    name = "update_one"


class UpdateMany(BaseDatabaseMethod):
    name = "update_many"


class RemoveOne(BaseDatabaseMethod):
    name = "remove_one"


class RemoveMany(BaseDatabaseMethod):
    name = "remove_many"




import enum

from AgentService.utils.jsonify import Jsonified
from AgentService.dtypes.message import Tool


class AgentResponseType(enum.Enum):
    tools = "tools"
    answer = "answer"


class AgentResponse(Jsonified):
    def __init__(self, type: AgentResponseType, content: str | list[Tool]):
        if isinstance(type, str):
            type = AgentResponseType(type)

        if isinstance(content, list):
            content = list(map(lambda x: Tool(**x) if isinstance(x, dict) else x, content))

        self.type = type
        self.content = content

        self.fields = ["type", "content"]

    def __str__(self):
        return "AgentResponse(type=" + self.type.value + ", content=" + str(self.content) + ")"




from AgentService.utils.jsonify import Jsonified


class BaseResponse(Jsonified):
    status: str = "base"
    description: str = "base"
    data: dict | object | None = None

    def __init__(self, description: str = None, data: dict | object | None = None):
        self.status = self.status
        self.description = description if description else self.description
        self.data = self.data if data is None else data

        self.fields = ["status", "description", "data"]

    def __str__(self):
        return f"{self.__class__.__name__}(description=\"{self.description}\", data={self.data})"

    def is_err(self):
        return self.status == "err"

    def is_ok(self):
        return self.status == "ok"


from .response import OkResponse, ErrResponse




import fastapi

from AgentService.config import Config
from AgentService.dtypes.response import OkResponse

from .models import SendMessage


chat_router = fastapi.APIRouter(prefix="/chat")


@chat_router.post("")
async def send_message(request: SendMessage):
    agent = Config().agent

    response = await agent.answer(
        chat_id=request.chat_id,
        text=request.text,
        context=request.context,
        tool_answers=list(map(lambda x: x.dict(), request.tool_answers))
    )

    return OkResponse(
        data=response
    ).to_dict()




from typing import Optional, Dict, List
from pydantic import BaseModel, Field


class ToolAnswer(BaseModel):
    id: str
    name: str
    text: Optional[str] = None


class SendMessage(BaseModel):
    chat_id: str
    text: Optional[str] = None
    context: Dict = Field(default_factory=dict)
    tool_answers: List[ToolAnswer] = Field(default_factory=list)
